==> OpenPatricianDisplay-0.6.0-RC1-sources.jar: ch/sahits/game/graphic/javafx/display/MainGameView.java <==
package ch.sahits.game.graphic.javafx.display;

import ch.sahits.game.event.EViewChangeEvent;
import ch.sahits.game.event.ViewChangeEvent;
import ch.sahits.game.event.data.SwitchCity;
import ch.sahits.game.graphic.display.ClientViewState;
import ch.sahits.game.graphic.display.EViewState;
import ch.sahits.game.graphic.display.dialog.util.DialogFactory;
import ch.sahits.game.graphic.display.gameplay.internal.EScene;
import ch.sahits.game.graphic.display.gameplay.internal.PolygonInitializerFactory;
import ch.sahits.game.graphic.display.model.ViewChangeCityPlayerProxyJFX;
import ch.sahits.game.graphic.image.IDataImageLoader;
import ch.sahits.game.graphic.image.IFontLoader;
import ch.sahits.game.graphic.image.IImageUtilities;
import ch.sahits.game.graphic.image.ImageScaleState;
import ch.sahits.game.graphic.image.model.ImageData;
import ch.sahits.game.javafx.model.EDialogType;
import ch.sahits.game.openpatrician.annotation.ClassCategory;
import ch.sahits.game.openpatrician.annotation.DialogCloseing;
import ch.sahits.game.openpatrician.annotation.DialogOpening;
import ch.sahits.game.openpatrician.annotation.EClassCategory;
import ch.sahits.game.openpatrician.annotation.ObjectPropertyType;
import ch.sahits.game.openpatrician.annotation.UniquePrototype;
import ch.sahits.game.openpatrician.dialog.Dialog;
import ch.sahits.game.openpatrician.dialog.DialogTemplate;
import ch.sahits.game.openpatrician.model.GameFactory;
import ch.sahits.game.openpatrician.model.IGame;
import ch.sahits.game.openpatrician.model.IHumanPlayer;
import ch.sahits.game.openpatrician.model.IMap;
import ch.sahits.game.openpatrician.model.ship.IConvoy;
import ch.sahits.game.openpatrician.model.ship.IShip;
import ch.sahits.game.openpatrician.spring.DependentAnnotationConfigApplicationContext;
import ch.sahits.game.openpatrician.spring.DialogScope;
import ch.sahits.game.openpatrician.util.IRebinabable;
import com.google.common.eventbus.AsyncEventBus;
import com.google.common.eventbus.Subscribe;
import javafx.application.Platform;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import javafx.scene.shape.Rectangle;
import lombok.Getter;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.ArrayList;
import java.util.List;

@UniquePrototype
@ClassCategory(EClassCategory.SINGLETON_BEAN)
public class MainGameView extends Group implements  IDialogContoller, IRebinabable {
	private static final Logger logger = LogManager.getLogger(MainGameView.class);
    public static final int MINMIMAL_DISPLAY_HEIGHT = 766;
    /**
	 * Use low level byte to indicate what is displayed instead of an enum.
	 */
	private double controlWidth;
	private double controlHeight;

	@Autowired
	private IImageUtilities imageUtils;
	@Autowired
	@Qualifier("mainScreenXMLImageLoader")
	private IDataImageLoader xmlLoader;
	@Autowired
	private IFontLoader fontLoader;
	@Autowired
	private DialogFactory dialogFactory;
	private Rectangle placeHolder;
	private BaseMainGameImageView imgView;
	@Getter
	private Dialog dialog;
    @Autowired
    private DialogScope dialogScope;
	@ObjectPropertyType(EScene.class)
	private ObjectProperty<EScene> currentScene = new SimpleObjectProperty<>(this, "currentScene", EScene.PORT);
    @Autowired
    private SceneEventHandlerFactory sceneEventHandlerFactory;
    @Autowired
    private PolygonInitializerFactory polygonInitFactory;
    @Autowired
    private ClientViewState viewState;
    @Autowired
    @Qualifier("clientEventBus")
    private AsyncEventBus clientEventBus;
	@Autowired
	private DisplayMessageOverlay dispMesgOverlay;
	@Autowired
	private DependentAnnotationConfigApplicationContext context;
	@Autowired
	private GameFactory gameFactory;
    @Autowired
    private IMap map;
    private StackPane stack;
    private SeamapImageView seaMapView;

    public MainGameView(double width, double heigth) {
		super();
		setManaged(false);
		controlWidth = width;
		controlHeight = heigth;
	}

	@PostConstruct
	private void createComponents() {

        setUpImageView();

        sceneEventHandlerFactory.currentSceneProperty().bindBidirectional(currentScene);
        sceneEventHandlerFactory.setUpDialogController(this);

        polygonInitFactory.currentSceneProperty().bindBidirectional(currentScene);

        stack = new StackPane();
		stack.getChildren().add(imgView);
		placeHolder = new Rectangle(controlWidth, controlHeight);
		placeHolder.setFill(Color.BLACK);
		dispMesgOverlay.setLayoutY(10);
		dispMesgOverlay.setLayoutX(10);
		getChildren().addAll(placeHolder, stack, dispMesgOverlay);
        clientEventBus.register(this);
		setOnMouseClicked(event -> {
			IGame game = gameFactory.getGame();
			game.normalSpeed();
		});
	}
	@PreDestroy
	private void unregister() {
		clientEventBus.unregister(this);
	}
	/**
	 * Retrieve the name for the image to be displayed in the scene
	 * @return
	 */
	private String getImageNameFromScene() {
		switch (currentScene.get()) {
		case MARKET:
			return "images/scene/marketPlaceScene";
		case PORT:
			return "images/scene/portScene";
		case SHIPYARD:
			return "images/scene/shipYard";
		case TAVERN:
			return "images/scene/tavernIterior";
		case CITY_HALL:
			return "images/scene/cityhall";
		case LOANER:
			return "images/scene/loaner";
		case CHURCH:
			return "images/scene/churchInterior";
		default:
			throw new RuntimeException(currentScene.get() + " is not implemented");
		}
	}

	private void setUpImageView() {
		String imageName = getImageNameFromScene();
		Image tmpImg = xmlLoader.getImage(imageName);
		ImageData imgData = xmlLoader.getImageData(imageName);
		Dimension2D targetDim = new Dimension2D(controlWidth, controlHeight);
		ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
		logger.debug("State of the port scene before: " + state);

        // // todo: andi 13/12/13: it would be simpler to bound the values instead of reinitializing them
        imgView = new MainGameImageView(controlWidth, controlHeight, imageUtils.cropAndScale(imageName, state), state);
		List<Polygon> polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
		imgView.resetPolygons(polygons);
        if (seaMapView != null) {
            seaMapView.setDisplayed(false);
        }
        viewState.setState(EViewState.CITY);

        logger.debug("State of the port scene after : " + state);
	}
    private void setUpSeamapImageView() {
        IHumanPlayer player = viewState.getPlayer();
        Image mapImage = imageUtils.createMapWithCities(map, player);
        Dimension2D origDim = new Dimension2D(mapImage.getWidth(), mapImage.getHeight());
        double scale = controlHeight/origDim.getHeight();
        Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
        mapImage = imageUtils.scale(mapImage, targetDim, scale);

        Point2D coordinates = viewState.getCurrentCityProxy().get().getCity().getCoordinates();
        coordinates = new Point2D(coordinates.getX() * scale, coordinates.getY() * scale);

        if (seaMapView == null) {
           seaMapView = (SeamapImageView) context.getBean("seamapImageView", new Object[]{mapImage, controlWidth, controlHeight, coordinates, scale});
        } else {
            seaMapView.resetImage(mapImage,controlWidth, controlHeight, scale);

        }
        seaMapView.setDisplayed(true);

        imgView = seaMapView;
        viewState.setState(EViewState.MAP);
    }

	public void widthChange(double oldWidth, double newWidth) {
        List<Polygon> polygons = new ArrayList<>();
        if (imgView instanceof MainGameImageView) {
            String imageName = getImageNameFromScene();
            Image tmpImg = xmlLoader.getImage(imageName);
            ImageData imgData = xmlLoader.getImageData(imageName);
            Dimension2D targetDim = new Dimension2D(newWidth, controlHeight);
            ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
            ImageView tmpImgView = imageUtils.cropAndScale(imageName, state);
            ((MainGameImageView) imgView).setImageView(newWidth, controlHeight, tmpImgView, state);
            polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
        }
        if (imgView instanceof SeamapImageView) {
            IHumanPlayer player = viewState.getPlayer();
            Image mapImage = imageUtils.createMapWithCities(map, player);
            Dimension2D origDim = new Dimension2D(mapImage.getWidth(), mapImage.getHeight());
            double scale = controlHeight/origDim.getHeight();
            Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
            mapImage = imageUtils.scale(mapImage, targetDim, scale);
            ((SeamapImageView)imgView).resetImage(mapImage,newWidth, controlHeight, scale);
        }
		imgView.resetPolygons(polygons);
		placeHolder.setWidth(newWidth);
		this.controlWidth = newWidth;
	}


	public void heightChange(double oldHeight, double newHeigth) {
        List<Polygon> polygons = new ArrayList<>();
        if (imgView instanceof MainGameImageView) {
            String imageName = getImageNameFromScene();
            Image tmpImg = xmlLoader.getImage(imageName);
            ImageData imgData = xmlLoader.getImageData(imageName);
            Dimension2D targetDim = new Dimension2D(controlWidth, newHeigth);
            ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
            ImageView tmpImgView = imageUtils.cropAndScale(imageName, state);
            ((MainGameImageView) imgView).setImageView(controlWidth, newHeigth, tmpImgView, state);
            polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
        }
        if (imgView instanceof SeamapImageView) {
            IHumanPlayer player = viewState.getPlayer();
            Image mapImage = imageUtils.createMapWithCities(map, player);
            Dimension2D origDim = new Dimension2D(mapImage.getWidth(), mapImage.getHeight());
            double scale = newHeigth/origDim.getHeight();
            Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
            mapImage = imageUtils.scale(mapImage, targetDim, scale);
            ((SeamapImageView)imgView).resetImage(mapImage,controlWidth, newHeigth, scale);
        }
        imgView.resetPolygons(polygons);
		placeHolder.setHeight(newHeigth);
		this.controlHeight = newHeigth;
	}
    @Subscribe
    public void handleViewChange(ViewChangeEvent event) {
        if (event.getAddresse().equals(MainGameView.class)) {
            if (event.getEventNotice() instanceof DialogTemplate) {
                 setNewDialog((DialogTemplate) event.getEventNotice());
            } else if (event.getEventNotice() instanceof ViewChangeCityPlayerProxyJFX
                    && ((ViewChangeCityPlayerProxyJFX)event.getEventNotice()).getViewChangeEvent() == EViewChangeEvent.MAIN_VIEW_SEA_MAP) {
                currentScene.set(EScene.SEAMAP);
                changeScene();
            } else {
                sceneEventHandlerFactory.getSceneEventHandler().handleEvent(event.getEventNotice());
            }
        }
    }

    /**
     * Replace the current dialog with a new one.
     * @param dialogType
     */
    @Override
    public void replaceDialog(EDialogType dialogType, Object...params) {
        closeDialog();
		while (dialog != null) {
			try {
				Thread.sleep(5);
			} catch (InterruptedException e) {
				// Do not care
			}
		}
        setNewDialog(dialogType, params);
    }

    /**
     * Close the currently open dialog
     */
    @Override
    @DialogCloseing
    public void closeDialog() {
		if (Platform.isFxApplicationThread()) {
			closeDialogUnwrapped();
		} else {
			Platform.runLater(() -> closeDialogUnwrapped());
		}
    }

	private void closeDialogUnwrapped() {
		if (dialog != null) {
            dialog.close();
			context.removePrototypeBean(dialog);
			getChildren().remove(dialog);
			dialog = null;
			dialogScope.closeScope();
		} else {
            logger.warn("Tried to close the dialog but it was already null");
        }
	}

	/**
     * Set up the new dialog.
     * @param dialogType
     */
    @Override
    @DialogOpening
    public void setNewDialog(EDialogType dialogType, Object...params) {
        if (params.length == 0) {
            dialog = dialogFactory.getDialog(dialogType, viewState.getCurrentCityProxy().get());
        } else if (params.length == 2) {
            if (params[0] instanceof IConvoy && params[1] instanceof IShip) {
                dialog = dialogFactory.getDialog(dialogType, viewState.getCurrentCityProxy().get(), (IConvoy)params[0], (IShip)params[1]);
            } else {
                throw new IllegalArgumentException("Cannot handle parameters "+params[0]+" and "+params[1]);
            }
        }
		dialog.setDialogType(dialogType);
		setNewDialog();
    }

    /**
     * Set up the new dialog.
     * @param template
     */
    @Override
    @DialogOpening
    public void setNewDialog(DialogTemplate template) {
        dialog = dialogFactory.getDialog(template);
        setNewDialog();
    }

    /**
     * Set up the dialog that was initialized. This method should only be called from #setNewDialog(EDialogType) or
     * #setNewDialog(DialogTemplate).
     */
    private void setNewDialog() {
        dialogScope.openScope();
        dialog.setLayoutX((controlWidth - Dialog.WIDTH) / 2);
        dialog.setLayoutY((controlHeight - Dialog.HEIGHT) / 2);
		if (Platform.isFxApplicationThread()) {
			getChildren().add(dialog);
		} else {
			Platform.runLater(() -> {
				if (dialog != null) {
					getChildren().add(dialog);
				}
			});
		}
    }

    /**
     * Change the scene image
     */
    @Override
    public void changeScene() {
		if (Platform.isFxApplicationThread()) {
			changeSceneUnwrapped();
		} else {
			Platform.runLater(() -> changeSceneUnwrapped());
		}
     }

	private void changeSceneUnwrapped() {
		closeDialog();
		stack.getChildren().removeAll(imgView);
        if (currentScene.get() == EScene.SEAMAP) {
            setUpSeamapImageView();
        } else {
            setUpImageView();
        }
        stack.getChildren().add(imgView);
    }

	@Override
	public void rebind() {
        if (currentScene.get() != EScene.PORT) {
            ViewChangeCityPlayerProxyJFX proxy = new ViewChangeCityPlayerProxyJFX(viewState.getCurrentCityProxy().get(), EViewChangeEvent.MAIN_VIEW_PORT);
            handleViewChange(new ViewChangeEvent(MainGameView.class, proxy));
        }
        if (seaMapView != null) {
            seaMapView.removeShipIcons();
        }
	}
	@Subscribe
	public void handleSwitchToCity(SwitchCity event) {
		currentScene.set(EScene.PORT);
		changeScene();
	}
}

==> OpenPatricianDisplay-0.7.0-RC1-sources.jar: ch/sahits/game/graphic/javafx/display/MainGameView.java <==
package ch.sahits.game.graphic.javafx.display;

import ch.sahits.game.event.EViewChangeEvent;
import ch.sahits.game.event.ViewChangeEvent;
import ch.sahits.game.event.data.SwitchCity;
import ch.sahits.game.graphic.display.ClientViewState;
import ch.sahits.game.graphic.display.EViewState;
import ch.sahits.game.graphic.display.dialog.util.DialogFactory;
import ch.sahits.game.graphic.display.gameplay.internal.EScene;
import ch.sahits.game.graphic.display.gameplay.internal.PolygonInitializerFactory;
import ch.sahits.game.graphic.display.model.ViewChangeCityPlayerProxyJFX;
import ch.sahits.game.graphic.image.IDataImageLoader;
import ch.sahits.game.graphic.image.IFontLoader;
import ch.sahits.game.graphic.image.IImageUtilities;
import ch.sahits.game.graphic.image.ImageScaleState;
import ch.sahits.game.graphic.image.model.ImageData;
import ch.sahits.game.javafx.model.EDialogType;
import ch.sahits.game.openpatrician.annotation.ClassCategory;
import ch.sahits.game.openpatrician.annotation.DialogCloseing;
import ch.sahits.game.openpatrician.annotation.DialogOpening;
import ch.sahits.game.openpatrician.annotation.EClassCategory;
import ch.sahits.game.openpatrician.annotation.ObjectPropertyType;
import ch.sahits.game.openpatrician.annotation.UniquePrototype;
import ch.sahits.game.openpatrician.clientserverinterface.model.factory.GameFactory;
import ch.sahits.game.openpatrician.dialog.Dialog;
import ch.sahits.game.openpatrician.dialog.DialogTemplate;
import ch.sahits.game.openpatrician.model.IGame;
import ch.sahits.game.openpatrician.model.IHumanPlayer;
import ch.sahits.game.openpatrician.model.map.IMap;
import ch.sahits.game.openpatrician.model.ship.IConvoy;
import ch.sahits.game.openpatrician.model.ship.IShip;
import ch.sahits.game.openpatrician.spring.DependentAnnotationConfigApplicationContext;
import ch.sahits.game.openpatrician.spring.DialogScope;
import ch.sahits.game.openpatrician.util.IRebinabable;
import com.google.common.eventbus.AsyncEventBus;
import com.google.common.eventbus.Subscribe;
import javafx.application.Platform;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import javafx.scene.shape.Rectangle;
import lombok.Getter;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.ArrayList;
import java.util.List;

@UniquePrototype
@ClassCategory(EClassCategory.SINGLETON_BEAN)
public class MainGameView extends Group implements  IDialogContoller, IRebinabable {
	private static final Logger logger = LogManager.getLogger(MainGameView.class);
    public static final int MINMIMAL_DISPLAY_HEIGHT = 766;
    /**
	 * Use low level byte to indicate what is displayed instead of an enum.
	 */
	private double controlWidth;
	private double controlHeight;

	@Autowired
	private IImageUtilities imageUtils;
	@Autowired
	@Qualifier("mainScreenXMLImageLoader")
	private IDataImageLoader xmlLoader;
	@Autowired
	private IFontLoader fontLoader;
	@Autowired
	private DialogFactory dialogFactory;
	private Rectangle placeHolder;
	private BaseMainGameImageView imgView;
	@Getter
	private Dialog dialog;
    @Autowired
    private DialogScope dialogScope;
	@ObjectPropertyType(EScene.class)
	private ObjectProperty<EScene> currentScene = new SimpleObjectProperty<>(this, "currentScene", EScene.PORT);
    @Autowired
    private SceneEventHandlerFactory sceneEventHandlerFactory;
    @Autowired
    private PolygonInitializerFactory polygonInitFactory;
    @Autowired
    private ClientViewState viewState;
    @Autowired
    @Qualifier("clientEventBus")
    private AsyncEventBus clientEventBus;
	@Autowired
	private DisplayMessageOverlay dispMesgOverlay;
	@Autowired
	private DependentAnnotationConfigApplicationContext context;
	@Autowired
	private GameFactory gameFactory;
    @Autowired
    private IMap map;
    private StackPane stack;
    private SeamapImageView seaMapView;

    public MainGameView(double width, double heigth) {
		super();
		setManaged(false);
		controlWidth = width;
		controlHeight = heigth;
	}

	@PostConstruct
	private void createComponents() {

        setUpImageView();

        sceneEventHandlerFactory.currentSceneProperty().bindBidirectional(currentScene);
        sceneEventHandlerFactory.setUpDialogController(this);

        polygonInitFactory.currentSceneProperty().bindBidirectional(currentScene);

        stack = new StackPane();
		stack.getChildren().add(imgView);
		placeHolder = new Rectangle(controlWidth, controlHeight);
		placeHolder.setFill(Color.BLACK);
		dispMesgOverlay.setLayoutY(10);
		dispMesgOverlay.setLayoutX(10);
		getChildren().addAll(placeHolder, stack, dispMesgOverlay);
        clientEventBus.register(this);
		setOnMouseClicked(event -> {
			IGame game = gameFactory.getGame();
			game.normalSpeed();
		});
	}
	@PreDestroy
	private void unregister() {
		clientEventBus.unregister(this);
	}
	/**
	 * Retrieve the name for the image to be displayed in the scene
	 * @return
	 */
	private String getImageNameFromScene() {
		switch (currentScene.get()) {
		case MARKET:
			return "images/scene/marketPlaceScene";
		case PORT:
			return "images/scene/portScene";
		case SHIPYARD:
			return "images/scene/shipYard";
		case TAVERN:
			return "images/scene/tavernIterior";
		case CITY_HALL:
			return "images/scene/cityhall";
		case LOANER:
			return "images/scene/loaner";
		case CHURCH:
			return "images/scene/churchInterior";
		default:
			throw new RuntimeException(currentScene.get() + " is not implemented");
		}
	}

	private void setUpImageView() {
		String imageName = getImageNameFromScene();
		Image tmpImg = xmlLoader.getImage(imageName);
		ImageData imgData = xmlLoader.getImageData(imageName);
		Dimension2D targetDim = new Dimension2D(controlWidth, controlHeight);
		ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
		logger.debug("State of the port scene before: " + state);

        // // todo: andi 13/12/13: it would be simpler to bound the values instead of reinitializing them
        imgView = new MainGameImageView(controlWidth, controlHeight, imageUtils.cropAndScale(imageName, state), state);
		List<Polygon> polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
		imgView.resetPolygons(polygons);
        viewState.setState(EViewState.CITY);

        logger.debug("State of the port scene after : " + state);
	}
    private void setUpSeamapImageView() {
        IHumanPlayer player = viewState.getPlayer();
        Image mapImage = imageUtils.createMapWithCities(map, player);
        Dimension2D origDim = new Dimension2D(mapImage.getWidth(), mapImage.getHeight());
        double scale = controlHeight/origDim.getHeight();
        Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
        mapImage = imageUtils.scale(mapImage, targetDim, scale);

        Point2D coordinates = viewState.getCurrentCityProxy().get().getCity().getCoordinates();
        coordinates = new Point2D(coordinates.getX() * scale, coordinates.getY() * scale);

        if (seaMapView == null) {
           seaMapView = (SeamapImageView) context.getBean("seamapImageView", new Object[]{mapImage, controlWidth, controlHeight, coordinates, scale});
        } else {
            seaMapView.resetImage(mapImage,controlWidth, controlHeight, scale);

        }

        imgView = seaMapView;
        viewState.setState(EViewState.MAP);
    }

	public void widthChange(double oldWidth, double newWidth) {
        List<Polygon> polygons = new ArrayList<>();

        Dimension2D origDim = map.getDimension();
        double scale = controlHeight/origDim.getHeight();
        if (imgView instanceof MainGameImageView) {
            String imageName = getImageNameFromScene();
            Image tmpImg = xmlLoader.getImage(imageName);
            ImageData imgData = xmlLoader.getImageData(imageName);
            Dimension2D targetDim = new Dimension2D(newWidth, controlHeight);
            ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
            ImageView tmpImgView = imageUtils.cropAndScale(imageName, state);
            ((MainGameImageView) imgView).setImageView(newWidth, controlHeight, tmpImgView, state);
            polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
        }
        if (imgView instanceof SeamapImageView) {

            Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
            IHumanPlayer player = viewState.getPlayer();
            Image mapImage = imageUtils.createMapWithCities(map, player);
            mapImage = imageUtils.scale(mapImage, targetDim, scale);
            ((SeamapImageView)imgView).resetImage(mapImage,newWidth, controlHeight, scale);
        }
		imgView.resetPolygons(polygons);
		placeHolder.setWidth(newWidth);
		this.controlWidth = newWidth;
	}


	public void heightChange(double oldHeight, double newHeigth) {
        List<Polygon> polygons = new ArrayList<>();
        Dimension2D origDim = map.getDimension();
        double scale = newHeigth/origDim.getHeight();
        if (imgView instanceof MainGameImageView) {
            String imageName = getImageNameFromScene();
            Image tmpImg = xmlLoader.getImage(imageName);
            ImageData imgData = xmlLoader.getImageData(imageName);
            Dimension2D targetDim = new Dimension2D(controlWidth, newHeigth);
            ImageScaleState state = new ImageScaleState(new Dimension2D(tmpImg.getWidth(), tmpImg.getHeight()), targetDim, imgData.getCrop(), imgData.getMaxCrop());
            ImageView tmpImgView = imageUtils.cropAndScale(imageName, state);
            ((MainGameImageView) imgView).setImageView(controlWidth, newHeigth, tmpImgView, state);
            polygons = polygonInitFactory.getScenePolygonInitializer().initialzePolygons(state);
        }
        if (imgView instanceof SeamapImageView) {
            Dimension2D targetDim = new Dimension2D(origDim.getWidth()*scale, origDim.getHeight()*scale);
            IHumanPlayer player = viewState.getPlayer();
            Image mapImage = imageUtils.createMapWithCities(map, player);
            mapImage = imageUtils.scale(mapImage, targetDim, scale);
            ((SeamapImageView)imgView).resetImage(mapImage,controlWidth, newHeigth, scale);
        }
        imgView.resetPolygons(polygons);
		placeHolder.setHeight(newHeigth);
		this.controlHeight = newHeigth;
	}
    @Subscribe
    public void handleViewChange(ViewChangeEvent event) {
        if (event.getAddresse().equals(MainGameView.class)) {
            if (event.getEventNotice() instanceof DialogTemplate) {
                 setNewDialog((DialogTemplate) event.getEventNotice());
            } else if (event.getEventNotice() instanceof ViewChangeCityPlayerProxyJFX
                    && ((ViewChangeCityPlayerProxyJFX)event.getEventNotice()).getViewChangeEvent() == EViewChangeEvent.MAIN_VIEW_SEA_MAP) {
                currentScene.set(EScene.SEAMAP);
                changeScene();
            } else {
                sceneEventHandlerFactory.getSceneEventHandler().handleEvent(event.getEventNotice());
            }
        }
    }

    /**
     * Replace the current dialog with a new one.
     * @param dialogType
     */
    @Override
    public void replaceDialog(EDialogType dialogType, Object...params) {
        closeDialog();
		while (dialog != null) {
			try {
				Thread.sleep(5);
			} catch (InterruptedException e) {
				// Do not care
			}
		}
        setNewDialog(dialogType, params);
    }

    /**
     * Close the currently open dialog
     */
    @Override
    @DialogCloseing
    public void closeDialog() {
		if (Platform.isFxApplicationThread()) {
			closeDialogUnwrapped();
		} else {
			Platform.runLater(() -> closeDialogUnwrapped());
		}
    }

	private void closeDialogUnwrapped() {
		if (dialog != null) {
            dialog.close();
			context.removePrototypeBean(dialog);
			getChildren().remove(dialog);
			dialog = null;
			dialogScope.closeScope();
		} else {
            logger.warn("Tried to close the dialog but it was already null");
        }
	}

	/**
     * Set up the new dialog.
     * @param dialogType
     */
    @Override
    @DialogOpening
    public void setNewDialog(EDialogType dialogType, Object...params) {
        if (params.length == 0) {
            dialog = dialogFactory.getDialog(dialogType, viewState.getCurrentCityProxy().get());
        } else if (params.length == 2) {
            if (params[0] instanceof IConvoy && params[1] instanceof IShip) {
                dialog = dialogFactory.getDialog(dialogType, viewState.getCurrentCityProxy().get(), (IConvoy)params[0], (IShip)params[1]);
            } else {
                throw new IllegalArgumentException("Cannot handle parameters "+params[0]+" and "+params[1]);
            }
        }
		dialog.setDialogType(dialogType);
		setNewDialog();
    }

    /**
     * Set up the new dialog.
     * @param template
     */
    @Override
    @DialogOpening
    public void setNewDialog(DialogTemplate template) {
        dialog = dialogFactory.getDialog(template);
        setNewDialog();
    }

    /**
     * Set up the dialog that was initialized. This method should only be called from #setNewDialog(EDialogType) or
     * #setNewDialog(DialogTemplate).
     */
    private void setNewDialog() {
        dialogScope.openScope();
        dialog.setLayoutX((controlWidth - Dialog.WIDTH) / 2);
        dialog.setLayoutY((controlHeight - Dialog.HEIGHT) / 2);
		if (Platform.isFxApplicationThread()) {
			getChildren().add(dialog);
		} else {
			Platform.runLater(() -> {
				if (dialog != null) {
					getChildren().add(dialog);
				}
			});
		}
    }

    /**
     * Change the scene image
     */
    @Override
    public void changeScene() {
		if (Platform.isFxApplicationThread()) {
			changeSceneUnwrapped();
		} else {
			Platform.runLater(() -> changeSceneUnwrapped());
		}
     }

	private void changeSceneUnwrapped() {
		closeDialog();
		stack.getChildren().removeAll(imgView);
        if (currentScene.get() == EScene.SEAMAP) {
            setUpSeamapImageView();
        } else {
            setUpImageView();
        }
        stack.getChildren().add(imgView);
    }

	@Override
	public void rebind() {
        if (currentScene.get() != EScene.PORT) {
            ViewChangeCityPlayerProxyJFX proxy = new ViewChangeCityPlayerProxyJFX(viewState.getCurrentCityProxy().get(), EViewChangeEvent.MAIN_VIEW_PORT);
            handleViewChange(new ViewChangeEvent(MainGameView.class, proxy));
        }
        if (seaMapView != null) {
            seaMapView.removeShipIcons();
        }
	}
	@Subscribe
	public void handleSwitchToCity(SwitchCity event) {
		currentScene.set(EScene.PORT);
		changeScene();
	}
}
